package com.gx.obe.struct.utils;

import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.bind.recursion.Recursions;
import com.gx.obe.common.file.utils.ParsingXmlUtils;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.AttachmentFile;
import com.gx.obe.struct.beans.BidderBasicInfo;
import com.gx.obe.struct.beans.BusinessLicense;
import com.gx.obe.struct.beans.Certificate;
import com.gx.obe.struct.beans.Finance;
import com.gx.obe.struct.beans.Performance;
import com.gx.obe.struct.beans.ProjectLeader;
import com.gx.obe.struct.beans.Qualification;
import com.gx.obe.struct.beans.TemplateDataItem;
import com.gx.obe.struct.beans.TemplateTable;
import com.gx.obe.struct.beans.WorkExperience;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.evaluation.EvaluationContent;
import com.gx.utils.BusinessNowTimeUtils;

/**
 * @Description: 解析结构化xml文件工具类
 * @author chenxw
 */
public class ParsingStructXmlUtils {
	
	private static final String BID_FILE_INDEX_TREE = "BidFileIndexTree.xml";
	
	private static final String TENDER_STRUCT = "TenderStruct.xml";
	
	private ParsingStructXmlUtils() {}
	
	private static Stream<Element> streamElement(List<?> list) {
		return list.stream().map(t -> (Element) t);
	}
	
	/**
	 * @Description: 解析评审内容
	 * @author chenxw
	 * @param tenderId
	 * @param factorCode
	 * @param evaluationContentListElement
	 * @return
	 */
	public static Optional<List<EvaluationContent>> resolverEvaluationContent(String tenderId, String factorCode, Element evaluationContentListElement) {
		return Optional.ofNullable(evaluationContentListElement).map(t -> t.elements("evaluationContent")).map(l -> {
			return streamElement(l).map(ParsingStructXmlUtils::initEvaluationContent).peek(t -> {
				t.setTenderId(tenderId);
				t.setFactorCode(factorCode);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化评审内容
	 * @author chenxw
	 * @param evaluationContentElement
	 * @return
	 */
	private static EvaluationContent initEvaluationContent(Element evaluationContentElement) {
		EvaluationContent evaluationContent = new EvaluationContent();
		evaluationContent.setId(UuidUtils.getUUID());
		evaluationContent.setRelChapterType(evaluationContentElement.elementTextTrim("relChapterType"));
		evaluationContent.setDataCategory(evaluationContentElement.elementTextTrim("dataCategory"));
		evaluationContent.setDataCode(evaluationContentElement.elementTextTrim("dataCode"));
		evaluationContent.setEvalRule(evaluationContentElement.elementTextTrim("evalRule"));
		evaluationContent.setTenderStructName(evaluationContentElement.elementTextTrim("tenderStructName"));
		evaluationContent.setEvalPointName(evaluationContentElement.elementTextTrim("evalPointName"));
		try {
			evaluationContent.setSortNo(Integer.valueOf(evaluationContentElement.elementTextTrim("Order")));
		} catch (Exception e) {}
		return evaluationContent;
	}
	
	/**
	 * @Description: 解析投标树形索引文件
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static Optional<List<Element>> resolverBidFileIndexTree(String bidFilePath) {
		return Optional.ofNullable(bidFilePath).map(t -> {
			return t.concat(BID_FILE_INDEX_TREE);
		}).map(XmlUtils::getXmlDocument).map(Document::getRootElement).map(Element::elements).map(l -> {
			return streamElement(l).map(t -> {
				List<Element> list = streamElement(t.elements()).collect(Collectors.toList());
				resolverBidFileIndexTree(list);
				return list;
			}).flatMap(List::stream).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 递归解析投标树形索引文件
	 * @author chenxw
	 * @param fileItemElementList
	 * @return
	 */
	private static void resolverBidFileIndexTree(List<Element> fileItemElementList) {
		List<Element> childfileItemList = streamElement(fileItemElementList).flatMap(t -> streamElement(t.elements("fileItem"))).collect(Collectors.toList());
		if (childfileItemList.isEmpty()) {
			return;
		}
		resolverBidFileIndexTree(childfileItemList);
		childfileItemList.stream().forEach(t -> fileItemElementList.add((Element) t.clone()));
	}
	
	/**
	 * @Description: 解析小范本表格
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @param relChapterType
	 * @param fileItemElementList
	 * @param bidFilePath
	 * @return
	 */
	public static Optional<List<TemplateTable>> resolverTemplateTable(String tenderId, String supplierId, String relChapterType, List<Element> fileItemElementList, String bidFilePath) {
		if (StringUtils.isEmpty(tenderId) || StringUtils.isEmpty(supplierId) || StringUtils.isEmpty(relChapterType) || CollectionUtils.isNull(fileItemElementList)) {
			return Optional.empty();
		}
		return fileItemElementList.stream().filter(t -> {
			return relChapterType.equals(t.elementTextTrim("chapterType"));
		}).findAny().map(t -> t.element("tableList")).map(t -> t.elements("tableInfo")).map(l -> {
			return streamElement(l).map(ParsingStructXmlUtils::initTemplateTable).peek(t -> {
				t.setTenderId(tenderId);
				t.setSupplierId(supplierId);
				t.setRelChapterType(relChapterType);
//				if (Stream.of("CompletedAchievements", "OngoingAchievements").anyMatch(Predicate.isEqual(relChapterType))) {
//					t.setDataCode(relChapterType);
//				}
				Optional.ofNullable(bidFilePath).map(p -> p.concat(t.getXmlPath())).map(ParsingXmlUtils::parsingTableXML).map(JsonUtil::jsonObjectToStr).ifPresent(t::setTableInfo);
			}).collect(Collectors.toList());
		});
	}
	
	public static Optional<List<TemplateTable>> resolverTemplateTable(String tenderId, String relChapterType) {
		String biddingFilePath = Constants.getBiddingFilePath(tenderId);
		Document document = XmlUtils.getXmlDocument(biddingFilePath + TENDER_STRUCT);
		
		if (document == null) {
			throw new RuntimeException();
		}
		List<?> elements = document.getRootElement().elements("chapter");
		if (elements == null) {
			return Optional.empty();
		}
		return Recursions.stream(ElementUtils.toStreamFun("chapterList", "chapter")).toStream(elements.stream().map(t -> (Element) t))
				.findAny(t -> t.elementText("relChapterType").equals(relChapterType)).map(ElementUtils.toStreamFun("tableList", "tableInfo"))
				.map(s -> {
					return s.map(ParsingStructXmlUtils::initTemplateTable).peek(t -> {
						t.setTenderId(tenderId);
						t.setSupplierId(Constants.BIDDING_ID);
						t.setRelChapterType(relChapterType);
						Optional.of(t).map(TemplateTable::getXmlPath).map(biddingFilePath::concat).map(ParsingXmlUtils::parsingTableXML).map(JsonUtil::jsonObjectToStr).ifPresent(t::setTableInfo);
					}).collect(Collectors.toList());
				});
	}
	
	/**
	 * @Description: 初始化小范本表格
	 * @author chenxw
	 * @param templateTableElement
	 * @return
	 */
	private static TemplateTable initTemplateTable(Element templateTableElement) {
		TemplateTable templateTable = new TemplateTable();
		String tableId = templateTableElement.elementTextTrim("tableId");
		templateTable.setId(StringUtils.isEmpty(tableId) ? UuidUtils.getUUID() : tableId);
		templateTable.setDataCode(templateTableElement.elementTextTrim("dataCode"));
		templateTable.setXmlPath(templateTableElement.elementTextTrim("xmlPath"));
		templateTable.setTableName(templateTableElement.elementTextTrim("tableName"));
		return templateTable;
	}
	
	/**
	 * @Description: 解析小范本表单数据
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @param relChapterType
	 * @param fileItemElementList
	 * @return
	 */
	public static Optional<List<TemplateDataItem>> resolverObeTemplateDataItem(String tenderId, String supplierId, String relChapterType, List<Element> fileItemElementList, String bidFilePath) {
		if (StringUtils.isEmpty(tenderId) || StringUtils.isEmpty(supplierId) || StringUtils.isEmpty(relChapterType) || CollectionUtils.isNull(fileItemElementList)) {
			return Optional.empty();
		}
		return fileItemElementList.stream().filter(t -> {
			return relChapterType.equals(t.elementTextTrim("chapterType"));
		}).findAny().map(t -> {
			return t.elementTextTrim("fileId");
		}).map(t -> {
			return t.concat(Constants.XML);
		}).map(t -> {
			return bidFilePath.concat(t);
		}).map(XmlUtils::getXmlDocument).map(Document::getRootElement).map(t -> t.elements("dataItem")).map(l -> {
			return streamElement(l).map(ParsingStructXmlUtils::initTemplateDataItem).peek(t -> {
				t.setTenderId(tenderId);
				t.setSupplierId(supplierId);
				t.setRelChapterType(relChapterType);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化小范本表格
	 * @author chenxw
	 * @param templateTableElement
	 * @return
	 */
	private static TemplateDataItem initTemplateDataItem(Element templateDataItemElement) {
		TemplateDataItem templateDataItem = new TemplateDataItem();
		templateDataItem.setId(UuidUtils.getUUID());
		templateDataItem.setDataCode(templateDataItemElement.elementTextTrim("id"));
		templateDataItem.setName(templateDataItemElement.elementTextTrim("name"));
		templateDataItem.setValue(templateDataItemElement.elementTextTrim("value"));
		templateDataItem.setType(templateDataItemElement.elementTextTrim("type"));
		templateDataItem.setDataType(templateDataItemElement.elementTextTrim("dataType"));
		templateDataItem.setUnit(templateDataItemElement.elementTextTrim("unit"));
		return templateDataItem;
	}
	
	/**
	 * @Description: 解析营业执照
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @param consumer
	 * @return
	 */
	public static Optional<List<BusinessLicense>> resolverBusinessLicense(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("BusinessLicense")).map(l -> {
			return streamElement(l).map(t -> {
				BusinessLicense businessLicense = initBusinessLicense(t);
				consumer.accept(new AttachmentFileOperation(tenderId, businessLicense.getId(), t.element("scanCertificate")));
				return businessLicense;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化营业执照
	 * @author chenxw
	 * @param businessLicenseElement
	 * @return
	 */
	private static BusinessLicense initBusinessLicense(Element businessLicenseElement) {
		BusinessLicense businessLicense = new BusinessLicense();
		businessLicense.setId(UuidUtils.getUUID());
		businessLicense.setRegisterNumber(businessLicenseElement.elementTextTrim("registerNumber"));
		businessLicense.setCompanyName(businessLicenseElement.elementTextTrim("companyName"));
		businessLicense.setCompanyType(businessLicenseElement.elementTextTrim("companyType"));
		businessLicense.setCompanyAddress(businessLicenseElement.elementTextTrim("companyAddress"));
		businessLicense.setLegalRepresentative(businessLicenseElement.elementTextTrim("legalRepresentative"));
		businessLicense.setRegisterCapital(businessLicenseElement.elementTextTrim("registerCapital"));
		businessLicense.setEstablishDate(businessLicenseElement.elementTextTrim("establishDate"));
		businessLicense.setBusnissTermStartDate(businessLicenseElement.elementTextTrim("busnissTermStartDate"));
		businessLicense.setBusnissTermEndtDate(businessLicenseElement.elementTextTrim("busnissTermEndDate"));
		businessLicense.setBusinessScope(businessLicenseElement.elementTextTrim("businessScope"));
		businessLicense.setRegisterInstitution(businessLicenseElement.elementTextTrim("registerInstitution"));
		businessLicense.setIsPerpetual(businessLicenseElement.elementTextTrim("isPerpetual"));
		businessLicense.setIsInAreaEnterprises(businessLicenseElement.elementTextTrim("isInAreaEnterprises"));
		return businessLicense;
	}
	
	/**
	 * @Description: 解析财务报表
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @param consumer
	 * @return
	 */
	public static Optional<List<Finance>> resolverFinance(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("Finance")).map(l -> {
			return streamElement(l).map(t -> {
				Finance finance = initFinance(t);
				consumer.accept(new AttachmentFileOperation(tenderId, finance.getId(), t.element("scanFinance")));
				return finance;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
//				t.getBalanceSheet().setTenderId(tenderId);
//				t.getBalanceSheet().setFinanceId(t.getId());
//				t.getProfitlossSheet().setTenderId(tenderId);
//				t.getProfitlossSheet().setFinanceId(t.getId());
//				t.getCashSheet().setTenderId(tenderId);
//				t.getCashSheet().setFinanceId(t.getId());
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化财务报表
	 * @author chenxw
	 * @param financeElement
	 * @return
	 */
	private static Finance initFinance(Element financeElement) {
		Finance finance = new Finance();
		finance.setId(UuidUtils.getUUID());
		finance.setAnnual(financeElement.elementTextTrim("annual"));
//		finance.setRegisteredCapital(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("registeredCapital")));
		finance.setTotal(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("total")));
		finance.setProfitLossProfit(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("profitLossprofit")));
		finance.setOperatingIncome(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("operatingIncome")));
		finance.setLiabilitiesRate(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("liabilitiesRate")));
		finance.setCurrentAssets(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("currentAssets")));
		finance.setCurrentLiabilities(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("currentLiabilities")));
		finance.setCurrentRate(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("currentRate")));
		finance.setProfitRatio(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("profitRatio")));
		finance.setOperatingMargin(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("operatingMargin")));
		finance.setRetainedProfits(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("retainedProfits")));
		finance.setCompanyName(financeElement.elementTextTrim("companyName"));
		finance.setNetWorth(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("netWorth")));
//		finance.setFixedAssets(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("fixedAssets")));
//		finance.setNetCashFlow(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("netCashFlow")));
//		finance.setReturnOnEquity(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("returnOnEquity")));
//		finance.setReturnOnTotalAsset(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("returnOnTotalAsset")));
		finance.setQuickRatio(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("quickRatio")));
		finance.setTotalliabilities(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("totalliabilities")));
		finance.setTotalcost(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("totalcost")));
		finance.setQuickassets(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("quickassets")));
		finance.setMainIncomeProfits(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("mainIncomeProfits")));
		finance.setPrimeOperatingRevenue(ObjectUtils.getObjBigDecimal(financeElement.elementTextTrim("primeOperatingRevenue")));
//		finance.setBalanceSheet(initBalanceSheet(financeElement.element("BalanceSheet")));
//		finance.setProfitlossSheet(initProfitlossSheet(financeElement.element("ProfitLossSheet")));
//		finance.setCashSheet(initCashSheet(financeElement.element("CashSheet")));
		return finance;
	}
	
	/**
	 * @Description: 初始化资产负债表
	 * @author chenxw
	 * @param balanceSheetElement
	 * @return
	 */
//	private static BalanceSheet initBalanceSheet(Element balanceSheetElement) {
//		BalanceSheet balanceSheet = new BalanceSheet();
//		balanceSheet.setId(UuidUtils.getUUID());
//		balanceSheet.setBalanceTotal(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("balanceTotal")));
//		balanceSheet.setBalanceCecurrentAssets(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("balanceCecurrentAssets")));
//		balanceSheet.setInventory(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("inventory")));
//		balanceSheet.setReceivables(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("receivables")));
//		balanceSheet.setBadAssets(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("badAssets")));
//		balanceSheet.setBalanceTotalliabilities(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("balanceTotalliabilities")));
//		balanceSheet.setBalanceCurrentLiabilities(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("balanceCurrentLiabilities")));
//		balanceSheet.setOwnershipInterest(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("ownershipInterest")));
//		balanceSheet.setAbnormalIncrease(ObjectUtils.getObjBigDecimal(balanceSheetElement.elementTextTrim("abnormalIncrease")));
//		return balanceSheet;
//	}
	
	/**
	 * @Description: 初始化损益表
	 * @author chenxw
	 * @param profitlossSheetElement
	 * @return
	 */
//	private static ProfitlossSheet initProfitlossSheet(Element profitlossSheetElement) {
//		ProfitlossSheet profitlossSheet = new ProfitlossSheet();
//		profitlossSheet.setId(UuidUtils.getUUID());
//		profitlossSheet.setMainIncome(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("mainIncome")));
//		profitlossSheet.setMainCost(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("mainCost")));
//		profitlossSheet.setFinancialCost(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("financialCost")));
//		profitlossSheet.setOtherCosts(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("otherCosts")));
//		profitlossSheet.setDevelopmentAndTransferCosts(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("developmentAndTransferCosts")));
//		profitlossSheet.setProfitLossprofit(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("profitLossprofit")));
//		profitlossSheet.setProfitRetainedProfits(ObjectUtils.getObjBigDecimal(profitlossSheetElement.elementTextTrim("profitRetainedProfits")));
//		return profitlossSheet;
//	}
	
	/**
	 * @Description: 初始化现金流量表
	 * @author chenxw
	 * @param cashSheetElement
	 * @return
	 */
//	private static CashSheet initCashSheet(Element cashSheetElement) {
//		CashSheet cashSheet = new CashSheet();
//		cashSheet.setId(UuidUtils.getUUID());
//		cashSheet.setOperatingNetCashFlow(ObjectUtils.getObjBigDecimal(cashSheetElement.elementTextTrim("operatingNetCashFlow")));
//		return cashSheet;
//	}
	
	/**
	 * @Description: 解析业绩
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @param consumer
	 * @param mark 
	 * @return
	 */
	public static Optional<List<Performance>> resolverPerformance(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer, String mark) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("Performance")).map(l -> {
			return streamElement(l).filter(t -> {
				if(null != mark ) {
					return mark.equals(t.elementTextTrim("status"));
				}else {
					return false;
				}
			}).map(t -> {
				Performance performance = initPerformance(t);
				consumer.accept(new AttachmentFileOperation(tenderId, performance.getId(), t.element("scanSingning")));
				return performance;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化业绩
	 * @author chenxw
	 * @param performanceElement
	 * @return
	 */
	private static Performance initPerformance(Element performanceElement) {
		Performance performance = new Performance();
		performance.setId(UuidUtils.getUUID());
		performance.setPurchasingUnit(performanceElement.elementTextTrim("purchasingUnit"));
		performance.setPartner(performanceElement.elementTextTrim("partner"));
		performance.setProjectName(performanceElement.elementTextTrim("projectName"));
		performance.setSingningDate(performanceElement.elementTextTrim("singningDate"));
		performance.setEndTime(performanceElement.elementTextTrim("endTime"));
		performance.setSingningTotal(performanceElement.elementTextTrim("singningTotal"));
		performance.setPurchaser(performanceElement.elementTextTrim("purchaser"));
		performance.setPurchaserPhone(performanceElement.elementTextTrim("purchaserPhone"));
		performance.setStatus(performanceElement.elementTextTrim("status"));
		performance.setSupplierName(performanceElement.elementTextTrim("supplierName"));
		performance.setProjectAddress(performanceElement.elementTextTrim("projectAddress"));
		performance.setBuyerName(performanceElement.elementTextTrim("buyerName"));
		performance.setBuyerAddress(performanceElement.elementTextTrim("buyerAddress"));
		performance.setBuyerPhoneNumber(performanceElement.elementTextTrim("buyerPhoneNumber"));
		performance.setStartTime(performanceElement.elementTextTrim("startTime"));
		performance.setWorkUndertaken(performanceElement.elementTextTrim("workUndertaken"));
		performance.setProjectQuantity(performanceElement.elementTextTrim("projectQuantity"));
		performance.setProjectManager(performanceElement.elementTextTrim("projectManager"));
		performance.setTechnicalDirector(performanceElement.elementTextTrim("technicalDirector"));
		performance.setChiefSupervisionEngineer(performanceElement.elementTextTrim("chiefSupervisionEngineer"));
		performance.setCsePhoneNumber(performanceElement.elementTextTrim("CSEPhoneNumber"));
		performance.setProjectDesc(performanceElement.elementTextTrim("projectDesc"));
		performance.setProjectRangeAndContent(performanceElement.elementTextTrim("projectRangeAndContent"));
		performance.setSupplierWorkerCountmax(performanceElement.elementTextTrim("supplierWorkerCountMax"));
		performance.setSupplierWorkerCountave(performanceElement.elementTextTrim("supplierWorkerCountAve"));
		performance.setProjectDeviceResult(performanceElement.elementTextTrim("projectDeviceResult"));
		performance.setBuyerLinkman(performanceElement.elementTextTrim("buyerLinkman"));
		performance.setIsHavrAccessory(performanceElement.elementTextTrim("isHavrAccessory"));
		performance.setIsOurCompanyProject(performanceElement.elementTextTrim("isOurCompanyProject"));
		performance.setServiceScope(performanceElement.elementTextTrim("serviceScope"));
		performance.setDeviceName(performanceElement.elementTextTrim("deviceName"));
		performance.setDevicetypeSpecification(performanceElement.elementTextTrim("devicetypeSpecification"));
		performance.setTotalCapacity(performanceElement.elementTextTrim("totalCapacity"));
		performance.setNumber(performanceElement.elementTextTrim("number"));
		performance.setContractTime(performanceElement.elementTextTrim("contractTime"));
		performance.setCommissioningTime(performanceElement.elementTextTrim("commissioningTime"));
		performance.setMemo(performanceElement.elementTextTrim("memo"));
		performance.setProjectScale(performanceElement.elementTextTrim("projectScale"));
		performance.setProfessionalNumber(performanceElement.elementTextTrim("professionalNumber"));
		performance.setProfessionalWorkload(performanceElement.elementTextTrim("professionalWorkload"));
		performance.setPartnerProfessionalWorkload(performanceElement.elementTextTrim("partnerProfessionalWorkload"));
		performance.setSupervisionCcontent(performanceElement.elementTextTrim("supervisionCcontent"));
		performance.setSupervisionResult(performanceElement.elementTextTrim("supervisionResult"));
		performance.setKeyStaffProfessionalAndDuty(performanceElement.elementTextTrim("keyStaffProfessionalAndDuty"));
		performance.setServiceDesc(performanceElement.elementTextTrim("serviceDesc"));
		performance.setExecutionDate(performanceElement.elementTextTrim("serviceDesc"));
		performance.setProjectScaleAndType(performanceElement.elementTextTrim("projectScaleAndType"));
		performance.setDesignStage(performanceElement.elementTextTrim("designStage"));
		performance.setDesignRange(performanceElement.elementTextTrim("designRange"));
		performance.setCooperationWay(performanceElement.elementTextTrim("cooperationWay"));
		performance.setContractExecuteCondition(performanceElement.elementTextTrim("contractExecuteCondition"));
		return performance;
	}
	
	/**
	 * @Description: 解析投标人基本情况
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @return
	 */
	public static Optional<List<BidderBasicInfo>> resolverBidderBasicInfo(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("bidderBasicInfo")).map(l -> {
			return streamElement(l).map(t -> {
				BidderBasicInfo bidderBasicInfo = initBidderBasicInfo(t);
				consumer.accept(new AttachmentFileOperation(tenderId, bidderBasicInfo.getId(), t.element("scanCertificate")));
				return bidderBasicInfo;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化投标人基本情况
	 * @author chenxw
	 * @param bidderBasicInfoElement
	 * @return
	 */
	private static BidderBasicInfo initBidderBasicInfo(Element bidderBasicInfoElement) {
		BidderBasicInfo bidderBasicInfo = new BidderBasicInfo();
		bidderBasicInfo.setId(UuidUtils.getUUID());
		bidderBasicInfo.setCompanyName(bidderBasicInfoElement.elementTextTrim("companyName"));
		bidderBasicInfo.setRegisterLocation(bidderBasicInfoElement.elementTextTrim("registerLocation"));
		bidderBasicInfo.setPostalCode(bidderBasicInfoElement.elementTextTrim("postalCode"));
		bidderBasicInfo.setLinkMan(bidderBasicInfoElement.elementTextTrim("linkMan"));
		bidderBasicInfo.setLinkManPhone(bidderBasicInfoElement.elementTextTrim("linkManPhone"));
		bidderBasicInfo.setLinkManFax(bidderBasicInfoElement.elementTextTrim("linkManFax"));
		bidderBasicInfo.setLinkManWebsite(bidderBasicInfoElement.elementTextTrim("linkManWebSite"));
		bidderBasicInfo.setOrgStructure(bidderBasicInfoElement.elementTextTrim("orgStructure"));
		bidderBasicInfo.setLegalRepresentativeName(bidderBasicInfoElement.elementTextTrim("legalRepresentativeName"));
		bidderBasicInfo.setLegalRepresentativeTitle(bidderBasicInfoElement.elementTextTrim("legalRepresentativeTitle"));
		bidderBasicInfo.setLegalRepresentativePhone(bidderBasicInfoElement.elementTextTrim("legalRepresentativePhone"));
		bidderBasicInfo.setTechnicalDirectorName(bidderBasicInfoElement.elementTextTrim("technicalDirectorName"));
		bidderBasicInfo.setTechnicalDirectorTitle(bidderBasicInfoElement.elementTextTrim("technicalDirectorTitle"));
		bidderBasicInfo.setTechnicalDirectorPhone(bidderBasicInfoElement.elementTextTrim("technicalDirectorPhone"));
		bidderBasicInfo.setSetupTime(bidderBasicInfoElement.elementTextTrim("setupTime"));
		bidderBasicInfo.setCompanyQualificationLevel(bidderBasicInfoElement.elementTextTrim("companyQualificationLevel"));
		bidderBasicInfo.setRegisteredCapital(bidderBasicInfoElement.elementTextTrim("registeredCapital"));
		bidderBasicInfo.setDepositBank(bidderBasicInfoElement.elementTextTrim("depositBank"));
		bidderBasicInfo.setBussinessLicense(bidderBasicInfoElement.elementTextTrim("bussinessLicense"));
		bidderBasicInfo.setBankAccount(bidderBasicInfoElement.elementTextTrim("bankAccount"));
		bidderBasicInfo.setEmployeeNumber(bidderBasicInfoElement.elementTextTrim("employeeNumber"));
		bidderBasicInfo.setPurchaserNumber(bidderBasicInfoElement.elementTextTrim("purchaserNumber"));
		bidderBasicInfo.setSeniorProfessionalPostNumber(bidderBasicInfoElement.elementTextTrim("seniorProfessionalPostNumber"));
		bidderBasicInfo.setMediumProfessionalPostNumber(bidderBasicInfoElement.elementTextTrim("mediumProfessionalPostNumber"));
		bidderBasicInfo.setPrimaryProfessionalPostNumber(bidderBasicInfoElement.elementTextTrim("primaryProfessionalPostNumber"));
		bidderBasicInfo.setArtisanNumber(bidderBasicInfoElement.elementTextTrim("artisanNumber"));
		bidderBasicInfo.setBusinessScope(bidderBasicInfoElement.elementTextTrim("businessScope"));
		bidderBasicInfo.setEnterpriseNature(bidderBasicInfoElement.elementTextTrim("enterpriseNature"));
		bidderBasicInfo.setEquityStructure(bidderBasicInfoElement.elementTextTrim("equityStructure"));
		bidderBasicInfo.setRelatedCompanyInfo(bidderBasicInfoElement.elementTextTrim("relatedCompanyInfo"));
		bidderBasicInfo.setServiceAbility(bidderBasicInfoElement.elementTextTrim("serviceAbility"));
		bidderBasicInfo.setPreInputDevice(bidderBasicInfoElement.elementTextTrim("preInputDevice"));
		bidderBasicInfo.setMemo(bidderBasicInfoElement.elementTextTrim("memo"));
		return bidderBasicInfo;
	}
	
	/**
	 * @Description: 解析认证证书
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @param consumer
	 * @return
	 */
	public static Optional<List<Qualification>> resolverQualification(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("Qualification")).map(l -> {
			return streamElement(l).map(t -> {
				Qualification qualification = initQualification(t);
				consumer.accept(new AttachmentFileOperation(tenderId, qualification.getId(), t.element("scanCertificate")));
				return qualification;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化认证证书
	 * @author chenxw
	 * @param qualificationElement
	 * @return
	 */
	private static Qualification initQualification(Element qualificationElement) {
		Qualification qualification = new Qualification();
		qualification.setId(UuidUtils.getUUID());
		qualification.setCertificateName(qualificationElement.elementTextTrim("certificateName"));
		qualification.setAwardProjectName(qualificationElement.elementTextTrim("awardProjectName"));
		qualification.setCompanyName(qualificationElement.elementTextTrim("companyName"));
		qualification.setAwardLevel(qualificationElement.elementTextTrim("awardLevel"));
		qualification.setCertificateNumber(qualificationElement.elementTextTrim("certificateNumber"));
		qualification.setIssueAuthority(qualificationElement.elementTextTrim("issueAuthority"));
		qualification.setIssueDate(qualificationElement.elementTextTrim("issueDate"));
		qualification.setPeriodValidity(qualificationElement.elementTextTrim("periodValidity"));
		qualification.setProjectName(qualificationElement.elementTextTrim("projectName"));
		return qualification;
	}
	
	/**
	 * @Description: 解析项目负责人
	 * @author chenxw
	 * @param tenderId
	 * @param modelDataId
	 * @param xmlFilePath
	 * @param consumer
	 * @return
	 */
	public static Optional<List<ProjectLeader>> resolverProjectLeader(String tenderId, String modelDataId, String xmlFilePath, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(XmlUtils.getXmlDocument(xmlFilePath)).map(t -> t.getRootElement().elements("projectLeader")).map(l -> {
			return streamElement(l).map(t -> {
				ProjectLeader projectLeader = initProjectLeader(t);
				consumer.accept(new AttachmentFileOperation(tenderId, projectLeader.getId(), t.element("scanSingningList")));
				projectLeader.setCertificateList(resolverCertificate(tenderId, projectLeader.getId(), t.element("certificateList"), consumer));
				projectLeader.setWorkExperienceList(resolverWorkExperience(t.element("workExperienceList"), tenderId, projectLeader.getId()));
				return projectLeader;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setModelDataId(modelDataId);
			}).collect(Collectors.toList());
		});
	}
	
	/**
	 * @Description: 初始化项目负责人
	 * @author chenxw
	 * @param qualificationElement
	 * @return
	 */
	private static ProjectLeader initProjectLeader(Element projectLeaderElement) {
		ProjectLeader projectLeader = new ProjectLeader();
		projectLeader.setId(UuidUtils.getUUID());
		projectLeader.setCompanyName(projectLeaderElement.elementTextTrim("companyName"));
		projectLeader.setLeaderName(projectLeaderElement.elementTextTrim("leaderName"));
		projectLeader.setIdnumber(projectLeaderElement.elementTextTrim("idnumber"));
		projectLeader.setAge(projectLeaderElement.elementTextTrim("age"));
		projectLeader.setJobTitle(projectLeaderElement.elementTextTrim("jobTitle"));
		projectLeader.setWorkExperience(projectLeaderElement.elementTextTrim("workExperience"));
		projectLeader.setCurrentMajorExperience(projectLeaderElement.elementTextTrim("currentMajorExperience"));
		projectLeader.setGraduateSchool(projectLeaderElement.elementTextTrim("graduateSchool"));
		projectLeader.setGraduateMajor(projectLeaderElement.elementTextTrim("graduateMajor"));
		projectLeader.setEducation(projectLeaderElement.elementTextTrim("education"));
		projectLeader.setProfessionalTitle(projectLeaderElement.elementTextTrim("professionalTitle"));
		projectLeader.setIsLeaderFlag(projectLeaderElement.elementTextTrim("isLeaderFlag"));
		return projectLeader;
	}
	
	/**
	 * @Description: 解析证书信息
	 * @author chenxw
	 * @param tenderId
	 * @param projectLeaderId
	 * @param certificateListElement
	 * @param consumer
	 * @return
	 */
	private static List<Certificate> resolverCertificate(String tenderId, String projectLeaderId, Element certificateListElement, Consumer<AttachmentFileOperation> consumer) {
		return Optional.ofNullable(certificateListElement).map(t -> t.elements("projectLeaderCertificate")).map(l -> {
			return streamElement(l).map(t -> {
				Certificate certificate = initCertificate(t);
				consumer.accept(new AttachmentFileOperation(tenderId, certificate.getId(), t.element("scanCertificate"), true));
				return certificate;
			}).peek(t -> {
				t.setTenderId(tenderId);
				t.setProjectLeaderId(projectLeaderId);
			}).collect(Collectors.toList());
		}).orElse(null);
	}
	
	/**
	 * @Description: 初始化证书信息
	 * @author chenxw
	 * @param projectLeaderCertificateElement
	 * @return
	 */
	private static Certificate initCertificate(Element projectLeaderCertificateElement) {
		Certificate certificate = new Certificate();
		certificate.setId(UuidUtils.getUUID());
		certificate.setCertificateNo(projectLeaderCertificateElement.elementTextTrim("certificateNo"));
		certificate.setCertificateName(projectLeaderCertificateElement.elementTextTrim("certificateName"));
		certificate.setCertificateLevel(projectLeaderCertificateElement.elementTextTrim("certificateLevel"));
		certificate.setSortNo(Optional.ofNullable(projectLeaderCertificateElement.elementTextTrim("SortNo")).map(Integer::valueOf).orElse(Integer.MAX_VALUE));
		return certificate;
	}
	
	/**
	 * @Description: 解析工作经历列表
	 * @author chenxw
	 * @param workExperienceListElement
	 * @param tenderId
	 * @param projectLeaderId
	 * @return 
	 */
	private static List<WorkExperience> resolverWorkExperience(
			Element workExperienceListElement, 
			String tenderId, 
			String projectLeaderId) {
		return Optional.ofNullable(workExperienceListElement)
				.map(Element::elements)
				.map(l -> IntStream.range(0, l.size())
						.mapToObj(i -> initWorkExperience((Element) l.get(i), i))
						.peek(t -> {
							t.setTenderId(tenderId);
							t.setProjectLeaderId(projectLeaderId);
						})
						.collect(Collectors.toList()))
				.orElse(null);
	}
	
	/**
	 * @Description: 初始化工作经历
	 * @author chenxw
	 * @param workExperienceElement
	 * @param
	 * @return
	 */
	private static WorkExperience initWorkExperience(Element workExperienceElement, int sortNo) {
		return Optional.of(workExperienceElement).map(e -> {
			WorkExperience workExperience = new WorkExperience();
			workExperience.setId(UuidUtils.getUUID());
			workExperience.setTime(e.elementTextTrim("time"));
			workExperience.setProjectName(e.elementTextTrim("projectName"));
			workExperience.setJobTitle(e.elementTextTrim("jobTitle"));
			workExperience.setContractInfo(e.elementTextTrim("contractInfo"));
			workExperience.setCapacity(e.elementTextTrim("capacity"));
			workExperience.setBearWork(e.elementTextTrim("bearWork"));
			workExperience.setSortNo(sortNo);
			return workExperience;
		}).get();
	}
	
	/**
	 * @Description: 解析附件信息
	 * @author chenxw
	 * @param attachmentFileOperation
	 * @return
	 */
	public static Optional<List<AttachmentFile>> resolverAttachmentFile(AttachmentFileOperation attachmentFileOperation) {
		Optional<AttachmentFileOperation> attachmentFileOperationOptional = Optional.ofNullable(attachmentFileOperation);
		Consumer<AttachmentFile> action = a -> {
			attachmentFileOperationOptional.map(t -> t.tenderId).ifPresent(t -> a.setTenderId(t));
			attachmentFileOperationOptional.map(t -> t.businessId).ifPresent(t -> a.setBusinessId(t));
		};
		if (attachmentFileOperation.single) return attachmentFileOperationOptional.map(t -> t.element)
				.map(e -> Stream.of(e).map(ParsingStructXmlUtils::initAttachmentFile).peek(action).collect(Collectors.toList()));
		return attachmentFileOperationOptional.map(t -> t.element).map(t -> t.elements("attachmentFileWithSign"))
				.map(l -> streamElement(l).map(ParsingStructXmlUtils::initAttachmentFile).peek(action).collect(Collectors.toList()));
	}
	
	/**
	 * @Description: 初始化附件信息
	 * @author chenxw
	 * @param attachmentFileWithSignElement
	 * @return
	 */
	private static AttachmentFile initAttachmentFile(Element attachmentFileWithSignElement) {
		AttachmentFile attachmentFile = new AttachmentFile();
		attachmentFile.setId(UuidUtils.getUUID());
		attachmentFile.setCreateTime(BusinessNowTimeUtils.getNowDate());
		attachmentFile.setFileUrl(attachmentFileWithSignElement.elementTextTrim("DownloadUrl"));
		attachmentFile.setFileName(attachmentFileWithSignElement.elementTextTrim("FileName"));
		return attachmentFile;
	}
	
	/**
	 * @Description: 附件信息操作类
	 * @author chenxw
	 */
	public static class AttachmentFileOperation {
		
		private final String tenderId;
		private final String businessId;
		private final Element element;
		private final boolean single;
		
		/** 
		 * @author chenxw
		 * @param tenderId
		 * @param businessId
		 * @param element 
		 */
		public AttachmentFileOperation(String tenderId, String businessId, Element element) {
			this(tenderId, businessId, element, false);
		}
		
		/** 
		 * @author chenxw
		 * @param tenderId
		 * @param businessId
		 * @param element
		 * @param single 
		 */
		public AttachmentFileOperation(String tenderId, String businessId, Element element, boolean single) {
			this.tenderId = tenderId;
			this.businessId = businessId;
			this.element = element;
			this.single = single;
		}
		
	}
	
}
